#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file was part of lago project.
# This file was part of INSPIRE-SCHEMAS.
# This file is part of autosemver.
# Copyright (C) 2014 Red Hat, Inc.
# Copyright (C) 2016 CERN.
# Copyright (C) 2016 David Caro.
#
# autosemver is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# autosemver is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE-SCHEMAS; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""
Script to generate the version, changelog and releasenotes from the git
repository.
"""
from __future__ import print_function

from functools import wraps

WITH_GIT = True
try:
    import dulwich.repo
    import dulwich.walk
except ImportError:
    WITH_GIT = False

from .git import (  # noqa
    _to_str,
    get_tags,
    get_refs,
    get_children_per_first_parent,
    get_repo_object,
    get_version,
    fuzzy_matches_refs,
    get_commit_type,
    pretty_commit,
)


def _needs_git(func):
    """
    Small decorator to make sure we have the git repo, or report error
    otherwise.
    """

    @wraps(func)
    def myfunc(*args, **kwargs):
        if not WITH_GIT:
            raise RuntimeError(
                "Dulwich library not available, can't extract info from the "
                "git repos."
            )
        return func(*args, **kwargs)

    return myfunc


@_needs_git
def get_changelog(repo_path, from_commit=None, bugtracker_url='',
                  rpm_format=False):
    """
    Given a repo path and an option commit/tag/refspec to start from, will
    get the rpm compatible changelog

    Args:
        repo_path (str): path to the git repo
        from_commit (str): refspec (partial commit hash, tag, branch, full
        refspec, partial refspec) to start the changelog from
        rpm_format(bool): if set, the changelog will be suitable to be uses as
        rpm package changelog.

    Returns:
        str: Rpm compatible changelog
    """
    repo = dulwich.repo.Repo(repo_path)
    tags = get_tags(repo)
    refs = get_refs(repo)
    changelog = []
    maj_version = 0
    feat_version = 0
    fix_version = 0
    start_including = False

    cur_line = ''
    if from_commit is None:
        start_including = True

    prev_version = (maj_version, feat_version, fix_version)

    for commit_sha, children in reversed(
        get_children_per_first_parent(repo_path).items()
    ):
        commit = get_repo_object(repo, commit_sha)
        maj_version, feat_version, fix_version = get_version(
            commit=commit,
            tags=tags,
            maj_version=maj_version,
            feat_version=feat_version,
            fix_version=fix_version,
            children=children,
        )
        version = (maj_version, feat_version, fix_version)
        version_str = '%s.%s.%s' % version

        if (
            start_including or commit_sha.startswith(from_commit) or
            fuzzy_matches_refs(from_commit, refs.get(commit_sha, []))
        ):
            commit_type = get_commit_type(
                commit=commit,
                children=children,
                tags=tags,
                prev_version=prev_version,
            )
            cur_line = pretty_commit(
                commit=commit,
                version=version_str,
                commit_type=commit_type,
                bugtracker_url=bugtracker_url,
                rpm_format=rpm_format,
            )
            for child in children:
                commit_type = get_commit_type(
                    commit=commit,
                    tags=tags,
                    prev_version=prev_version,
                )
                cur_line += pretty_commit(
                    commit=child,
                    version=None,
                    commit_type=commit_type,
                    bugtracker_url=bugtracker_url,
                )
            start_including = True
            changelog.append(cur_line)

        prev_version = version

    return '\n'.join(reversed(changelog))


@_needs_git
def get_current_version(repo_path):
    """
    Given a repo will return the version string, according to semantic
    versioning, counting as non-backwards compatible commit any one with a
    message header that matches (case insensitive)::

        sem-ver: .*break.*

    And as features any commit with a header matching::

        sem-ver: feature

    And counting any other as a bugfix
    """
    repo = dulwich.repo.Repo(repo_path)
    tags = get_tags(repo)
    maj_version = 0
    feat_version = 0
    fix_version = 0

    for commit_sha, children in reversed(
            get_children_per_first_parent(repo_path).items()
    ):
        commit = get_repo_object(repo, commit_sha)
        maj_version, feat_version, fix_version = get_version(
            commit=commit,
            tags=tags,
            maj_version=maj_version,
            feat_version=feat_version,
            fix_version=fix_version,
            children=children,
        )

    return '%s.%s.%s' % (maj_version, feat_version, fix_version)


@_needs_git
def get_authors(repo_path, from_commit=None):
    """
    Given a repo and optionally a base revision to start from, will return
    the list of authors.
    """
    repo = dulwich.repo.Repo(repo_path)
    refs = get_refs(repo)
    start_including = False
    authors = set()

    if from_commit is None:
        start_including = True

    for commit_sha, children in reversed(
        get_children_per_first_parent(repo_path).items()
    ):
        commit = get_repo_object(repo, commit_sha)
        if (
            start_including or commit_sha.startswith(from_commit) or
            fuzzy_matches_refs(from_commit, refs.get(commit_sha, []))
        ):
            authors.add(_to_str(commit.author))
            for child in children:
                authors.add(_to_str(child.author))

            start_including = True

    return sorted(authors)


@_needs_git
def get_releasenotes(repo_path, from_commit=None, bugtracker_url=''):
    """
    Given a repo and optionally a base revision to start from, will return
    a text suitable for the relase notes announcement, grouping the bugs, the
    features and the api-breaking changes.
    """
    repo = dulwich.repo.Repo(repo_path)
    tags = get_tags(repo)
    refs = get_refs(repo)
    maj_version = 0
    feat_version = 0
    fix_version = 0
    start_including = False
    bugs = []
    features = []
    api_break_changes = []

    cur_line = ''
    if from_commit is None:
        start_including = True

    prev_version = (maj_version, feat_version, fix_version)

    for commit_sha, children in reversed(
        get_children_per_first_parent(repo_path).items()
    ):
        commit = get_repo_object(repo, commit_sha)
        maj_version, feat_version, fix_version = get_version(
            commit=commit,
            tags=tags,
            maj_version=maj_version,
            feat_version=feat_version,
            fix_version=fix_version,
            children=children,
        )
        version = (maj_version, feat_version, fix_version)
        version_str = '%s.%s.%s' % version

        if (
            start_including or commit_sha.startswith(from_commit) or
            fuzzy_matches_refs(from_commit, refs.get(commit_sha, []))
        ):
            parent_commit_type = get_commit_type(
                commit=commit,
                children=children,
                tags=tags,
                prev_version=prev_version,
            )
            cur_line = pretty_commit(
                commit=commit,
                version=version_str,
                bugtracker_url=bugtracker_url,
            )
            for child in children:
                commit_type = get_commit_type(
                    commit=commit,
                    tags=tags,
                    prev_version=prev_version,
                )
                cur_line += pretty_commit(
                    commit=child,
                    version=None,
                    commit_type=commit_type,
                    bugtracker_url=bugtracker_url,
                )
            start_including = True

            if parent_commit_type == 'api_break':
                api_break_changes.append(cur_line)
            elif parent_commit_type == 'feature':
                features.append(cur_line)
            else:
                bugs.append(cur_line)

        prev_version = version

    return u'''
New changes for version %s
=================================

API Breaking changes
--------------------
%s

New features
------------
%s

Bugfixes and minor changes
--------------------------
%s
''' % (
            version_str,
            '\n'.join(reversed(api_break_changes)),
            '\n'.join(reversed(features)),
            '\n'.join(reversed(bugs)),
        )



# -*- coding: utf-8 -*-
#
# This file was part of lago project.
# This file was part of INSPIRE-SCHEMAS.
# This file is part of autosemver.
# Copyright (C) 2016 CERN.
#
# autosemver is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# autosemver is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with autosemver; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

import copy
import sys

import argparse

from .api import (
    get_authors,
    get_changelog,
    get_current_version,
    get_releasenotes,
)
from .git import _to_str
from .packaging import (
    get_current_version as pkg_version,
    create_authors,
    create_changelog,
)

PROJECT_NAME = 'python-autosemver'


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser()
    parser.add_argument(
        'repo_path', help='Git repo to generate the changelog for.'
    )
    subparsers = parser.add_subparsers()
    changelog_parser = subparsers.add_parser('changelog')
    changelog_parser.add_argument(
        '--from-commit',
        default=None,
        help='Commit to start the changelog from'
    )
    changelog_parser.add_argument(
        '--rpm-format',
        action='store_true',
        help='If set, the changelog will be rpm friendly.'
    )
    changelog_parser.set_defaults(func=get_changelog)
    version_parser = subparsers.add_parser('version')
    version_parser.set_defaults(func=get_current_version)
    releasenotes_parser = subparsers.add_parser('releasenotes')
    releasenotes_parser.add_argument(
        '--from-commit',
        default=None,
        help='Commit to start the release notes from.'
    )
    releasenotes_parser.set_defaults(func=get_releasenotes)
    authors_parser = subparsers.add_parser('authors')
    authors_parser.add_argument(
        '--from-commit',
        default=None,
        help='Commit to start the authors from.'
    )
    authors_parser.set_defaults(
        func=lambda *args, **kwargs: '\n'.join(get_authors(*args, **kwargs))
    )
    args = parser.parse_args(args)

    params = copy.deepcopy(vars(args))
    params.pop('func')

    print(_to_str(args.func(**params)))


def distutils(dist, attr, value):
    if attr != 'autosemver':
        if value:
            setattr(dist.metadata, attr, value)
            if getattr(dist.metadata, '_autosmever', ''):
                create_changelog(bugtracker_url=value)

        return

    dist.metadata.version = pkg_version()
    create_authors()

    create_changelog(
        bugtracker_url=getattr(dist.metadata, 'bugtracker_url', ''),
    )

    dist.metadata._autosmever = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file was part of lago project.
# This file was part of INSPIRE-SCHEMAS.
# This file is part of autosemver.
# Copyright (C) 2014 Red Hat, Inc.
# Copyright (C) 2016 CERN.
# Copyright (C) 2016 David Caro.
#
# autosemver is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# autosemver is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with autosemver; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""
Script to generate the version, changelog and releasenotes from the git
repository.
"""
from __future__ import print_function

import datetime
import os
import re
from collections import OrderedDict, defaultdict

import dulwich.repo
import dulwich.walk

BUG_URL_REG = re.compile(
    r'.*(closes #|fixes #|adresses #)(?P<bugid>\d+)'
)
VALID_TAG = re.compile(r'^\d+\.\d+$')
FEAT_HEADER = re.compile(
    r'\nsem-ver:\s*.*(feature|deprecat).*\n',
    flags=re.IGNORECASE,
)
FEAT_MSG = re.compile(r'\n\* NEW')
MAJOR_HEADER = re.compile(r'\nsem-ver:\s*.*break.*\n', flags=re.IGNORECASE)
MAJOR_MSG = re.compile(r'\n\* INCOMPATIBLE')


def _to_str(maybe_str):
    # python3 support magic
    try:
        assert isinstance(u'', str)
        unicode = type(None)
    except AssertionError:
        unicode = type(u'')

    if isinstance(maybe_str, unicode):
        maybe_str = maybe_str.encode('utf-8')

    else:
        try:
            maybe_str = maybe_str.decode('utf-8')
        except (UnicodeDecodeError, UnicodeEncodeError, AttributeError):
            pass

    return maybe_str


def get_repo_object(repo, object_name):
    try:
        object_name = object_name.encode()
    except:
        pass

    return repo.get_object(object_name)


def fit_to_cols(what, indent, cols=79):
    lines = []
    free_cols = cols - len(indent)
    has_indent = False
    while len(what) > free_cols and ' ' in what.lstrip():
        cutpoint = free_cols
        extra_indent = ''
        if what[free_cols] != ' ':
            try:
                if has_indent:
                    prev_space = what[len(has_indent):free_cols].rindex(' ')
                else:
                    prev_space = what[:free_cols].rindex(' ')

                lines.append(indent + what[:prev_space])
                cutpoint = prev_space + 1
                extra_indent = '          '
            except ValueError:
                lines.append(indent + what[:free_cols] + '-')
        else:
            lines.append(indent + what[:free_cols])
        what = extra_indent + what[cutpoint:]
        if extra_indent:
            has_indent = extra_indent
    lines.append(indent + what)
    return '\n'.join(lines)


def get_bugs_from_commit_msg(commit_msg):
    bugs = []
    for line in _to_str(commit_msg).split('\n'):
        match = BUG_URL_REG.match(_to_str(line))
        if match:
            bugs.append(match.groupdict()['bugid'])
    return bugs


def pretty_commit(commit, version=None, commit_type='bug', bugtracker_url='',
                  rpm_format=False):
    message = _to_str(commit.message)
    subject = _to_str(commit.message).split('\n', 1)[0]  # noqa
    short_hash = commit.sha().hexdigest()[:8]  # noqa
    author = _to_str(commit.author)  # noqa
    author_date = datetime.datetime.fromtimestamp(  # noqa
            int(commit.commit_time)
    ).strftime('%a %b %d %Y')
    bugs = get_bugs_from_commit_msg(commit.message)
    if bugs:
        changelog_bugs = fit_to_cols(
            'FIXED ISSUES: ' + ', '.join(
                '{bugtracker_url}{bug}'.format(
                    bugtracker_url=bugtracker_url,
                    bug=bug,
                ) for bug in bugs
            ),
            indent='    ',
        ) + '\n'
    else:
        changelog_bugs = ''  # noqa

    feature_header = ''
    if commit_type == 'feature':
        feature_header = 'FEATURE'
    elif commit_type == 'api_break':
        feature_header = 'MAJOR'
    else:
        feature_header = 'MINOR'

    changelog_message = fit_to_cols(  # noqa
        u'{feature_header} {short_hash}: {subject}'.format(**vars()),
        indent='    ',
    )

    if rpm_format:
        return (
            (
                u'* {author_date} {author} - {version}\n'
                if version is not None else ''
            ) + '{changelog_message}\n' + '{changelog_bugs}'
        ).format(**vars())

    return (
        (
            u'* {version} "{author}"\n'
            if version is not None else ''
        ) + '{changelog_message}\n' + '{changelog_bugs}'
    ).format(**vars())


def get_tags(repo):
    tags = {}
    for tag_ref, commit in repo.get_refs().items():
        tag_ref = _to_str(tag_ref)
        if tag_ref.startswith('refs/tags/') and VALID_TAG.match(
            tag_ref[len('refs/tags/'):]
        ):
            tags[_to_str(commit)] = os.path.basename(tag_ref)

    return tags


def get_refs(repo):
    refs = defaultdict(set)
    for ref, commit in repo.get_refs().items():
        refs[commit].add(commit)
        refs[commit].add(ref)
    return refs


def fuzzy_matches_ref(fuzzy_ref, ref):
    cur_section = ''
    for path_section in reversed(_to_str(ref).split('/')):
        cur_section = os.path.normpath(os.path.join(path_section, cur_section))
        if fuzzy_ref == cur_section:
            return True
    return False


def fuzzy_matches_refs(fuzzy_ref, refs):
    return any(fuzzy_matches_ref(fuzzy_ref, ref) for ref in refs)


def get_children_per_parent(repo_path):
    repo = dulwich.repo.Repo(repo_path)
    children_per_parent = defaultdict(set)

    for entry in repo.get_walker(order=dulwich.walk.ORDER_TOPO):
        for parent in entry.commit.parents:
            children_per_parent[_to_str(parent)].add(
                entry.commit.sha().hexdigest()
            )

    return children_per_parent


def get_first_parents(repo_path):
    repo = dulwich.repo.Repo(repo_path)
    #: these are the commits that are parents of more than one other commit
    first_parents = []
    on_merge = False

    for entry in repo.get_walker(order=dulwich.walk.ORDER_TOPO):
        commit = entry.commit
        # In order to properly work on python 2 and 3 we need some utf magic
        parents = commit.parents and [_to_str(i) for i in commit.parents]
        if not parents:
            if commit.sha().hexdigest() not in first_parents:
                first_parents.append(commit.sha().hexdigest())
        elif len(parents) == 1 and not on_merge:
            if commit.sha().hexdigest() not in first_parents:
                first_parents.append(commit.sha().hexdigest())
            if parents[0] not in first_parents:
                first_parents.append(parents[0])
        elif len(parents) > 1 and not on_merge:
            on_merge = True
            if commit.sha().hexdigest() not in first_parents:
                first_parents.append(commit.sha().hexdigest())
            if parents[0] not in first_parents:
                first_parents.append(parents[0])
        elif parents and commit.sha().hexdigest() in first_parents:
            if parents[0] not in first_parents:
                first_parents.append(parents[0])

    return first_parents


def has_firstparent_child(sha, first_parents, parents_per_child):
    return any(
        child for child in parents_per_child[sha] if child in first_parents
    )


def get_merged_commits(repo, commit, first_parents, children_per_parent):
    merge_children = set()

    to_explore = set([commit.sha().hexdigest()])

    while to_explore:
        next_sha = to_explore.pop()
        try:
            next_commit = get_repo_object(repo, next_sha)
        except KeyError:
            continue

        if (
            next_sha not in first_parents and not has_firstparent_child(
                next_sha, first_parents, children_per_parent
            ) or next_sha.encode('utf-8') in commit.parents
        ):
            merge_children.add(next_sha)

        non_first_parents = (
            parent
            for parent in next_commit.parents
            if _to_str(parent) not in first_parents
        )
        for child_sha in non_first_parents:
            if child_sha not in merge_children and child_sha != next_sha:
                to_explore.add(child_sha)

    return merge_children


def get_children_per_first_parent(repo_path):
    repo = dulwich.repo.Repo(repo_path)
    first_parents = get_first_parents(repo_path)
    children_per_parent = get_children_per_parent(repo_path)
    children_per_first_parent = OrderedDict()

    for first_parent in first_parents:
        try:
            commit = get_repo_object(repo, first_parent)
        except KeyError:
            continue

        if len(commit.parents) > 1:
            children = get_merged_commits(
                repo=repo,
                commit=commit,
                first_parents=first_parents,
                children_per_parent=children_per_parent,
            )
        else:
            children = set()

        children_per_first_parent[first_parent] = [
            get_repo_object(repo, child) for child in children
        ]

    return children_per_first_parent


def get_version(commit, tags, maj_version=0, feat_version=0, fix_version=0,
                children=None):
    children = children or []
    commit_type = get_commit_type(commit, children)
    commit_sha = commit.sha().hexdigest()

    if commit_sha in tags:
        maj_version, feat_version = tags[commit_sha].split('.')
        maj_version = int(maj_version)
        feat_version = int(feat_version)
        fix_version = 0
    elif commit_type == 'api_break':
        maj_version += 1
        feat_version = 0
        fix_version = 0
    elif commit_type == 'feature':
        feat_version += 1
        fix_version = 0
    else:
        fix_version += 1

    version = (maj_version, feat_version, fix_version)
    return version


def is_api_break(commit):
    return (
        MAJOR_HEADER.search(_to_str(commit.message)) or
        MAJOR_MSG.search(_to_str(commit.message))
    )


def is_feature(commit):
    return (
        FEAT_HEADER.search(_to_str(commit.message)) or
        FEAT_MSG.search(_to_str(commit.message))
    )


def get_commit_type(commit, children=None, tags=None, prev_version=None):
    children = children or []
    tags = tags or []
    prev_version = prev_version or (0, 0, 0)
    commit_sha = commit.sha().hexdigest()

    if commit_sha in tags:
        maj_version, feat_version = tags[commit_sha].split('.')
        maj_version = int(maj_version)
        feat_version = int(feat_version)
        if maj_version > prev_version[0]:
            return 'api_break'
        elif feat_version > prev_version[1]:
            return 'feature'
        return 'bug'

    if any(is_api_break(child) for child in children + [commit]):
        return 'api_break'
    elif any(is_feature(child) for child in children + [commit]):
        return 'feature'
    else:
        return 'bug'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file was part of lago project.
# This file was part of INSPIRE-SCHEMAS.
# This file is part of autosemver.
# Copyright (C) 2014 Red Hat, Inc.
# Copyright (C) 2016 CERN.
# Copyright (C) 2016 David Caro.
#
# autosemver is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# autosemver is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with autosemver; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.
import os
import pkg_resources

from . import api


def get_current_version(project_name=None, project_dir=os.curdir,
                        repo_dir=None):
    """
    Retrieves the version of the package, checking in this order of priority:

    * From an environment variable named ${project_name}_VERSION (all in caps)
      if project_name was specified.
    * From the PKG-INFO file if inside a packaged distro.
    * From the git history.

    Args:
        project_name(str): Name of the project to get the version for, if none
            passed, will not use any environment variable override.

    Returns:
        str: Version for the package.

    Raises:
        RuntimeError: If the version could not be retrieved.
    """
    if project_name is not None:
        version_env_var = '%s_VERSION' % project_name.upper()

        if version_env_var in os.environ and os.environ[version_env_var]:
            return os.environ[version_env_var]

    version = None
    repo_dir = repo_dir or project_dir
    pkg_info_file = os.path.join(project_dir, 'PKG-INFO')
    if os.path.exists(pkg_info_file):
        with open(pkg_info_file) as info_fd:
            for line in info_fd.readlines():
                if line.startswith('Version: '):
                    version = line.split(' ', 1)[-1]

    if version is None:
        try:
            version = api.get_current_version(repo_path=repo_dir)
        except:
            pass

    if version is None:
        if project_name:
            try:
                version = (
                    pkg_resources.get_distribution(project_name).split(' ')[0]
                )
            except:
                pass

    if version is None:
        raise RuntimeError('Failed to get package version')

    # py3 compatibility step
    if not isinstance(version, str) and isinstance(version, bytes):
        version = api._to_str(version)

    return version


def get_authors(project_dir=os.curdir):
    """
    Retrieves the authors list, from the AUTHORS file (if in a package) or
    generates it from the git history.

    Returns:
        list(str): List of authors

    Raises:
        RuntimeError: If the authors could not be retrieved
    """
    authors = set()
    pkg_info_file = os.path.join(project_dir, 'PKG-INFO')
    authors_file = os.path.join(project_dir, 'AUTHORS')
    if os.path.exists(pkg_info_file) and os.path.exists(authors_file):
        with open(authors_file) as authors_fd:
            authors = set(authors_fd.read().splitlines())
    else:
        authors = api.get_authors(repo_path=project_dir)

    return authors


def get_changelog(project_dir=os.curdir, bugtracker_url='', rpm_format=False):
    """
    Retrieves the changelog, from the CHANGELOG file (if in a package) or
    generates it from the git history. Optionally in rpm-compatible format.

    Args:
        project_dir(str): Path to the git repo of the project.
        bugtracker_url(str): Url to the bug tracker for the issues.
        rpm_format(bool): if set to True, will make the changelog
            rpm-compatible

    Returns:
        str: changelog

    Raises:
        RuntimeError: If the changelog could not be retrieved
    """
    changelog = ''
    pkg_info_file = os.path.join(project_dir, 'PKG-INFO')
    changelog_file = os.path.join(project_dir, 'CHANGELOG')
    if os.path.exists(pkg_info_file) and os.path.exists(changelog_file):
        with open(changelog_file) as changelog_fd:
            changelog = changelog_fd.read()

    else:
        changelog = api.get_changelog(
            repo_path=project_dir,
            bugtracker_url=bugtracker_url,
            rpm_format=rpm_format,
        )

    return changelog


def create_authors(project_dir=os.curdir):
    """
    Creates the authors file, if not in a package.

    Returns:
        None

    Raises:
        RuntimeError: If the authors could not be retrieved
    """
    pkg_info_file = os.path.join(project_dir, 'PKG-INFO')
    authors_file = os.path.join(project_dir, 'AUTHORS')
    if os.path.exists(pkg_info_file):
        return

    authors = get_authors(project_dir=project_dir)
    with open(authors_file, 'wb') as authors_fd:
        authors_fd.write(
            b'\n'.join(a.encode('utf-8') for a in authors)
        )


def create_changelog(project_dir=os.curdir, bugtracker_url='',
                     rpm_format=False):
    """
    Creates the changelog file, if not in a package.

    Args:
        project_dir(str): Path to the git repo of the project.
        bugtracker_url(str): Url to the bug tracker for the issues.
        rpm_format(bool): if set to True, will make the changelog
            rpm-compatible

    Returns:
        None

    Raises:
        RuntimeError: If the changelog could not be retrieved
    """
    pkg_info_file = os.path.join(project_dir, 'PKG-INFO')
    if os.path.exists(pkg_info_file):
        return

    with open('CHANGELOG', 'wb') as changelog_fd:
        changelog_fd.write(
            get_changelog(
                project_dir=project_dir,
                bugtracker_url=bugtracker_url,
                rpm_format=rpm_format,
            ).encode('utf-8')
        )



# -*- coding: utf-8 -*-
#
# This file was part of lago project.
# This file was part of INSPIRE-SCHEMAS.
# This file is part of autosemver.
# Copyright (C) 2016 CERN.
#
# autosemver is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# autosemver is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with autosemver; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.
"""Automatic semantic versioning from git history for python packages"""

import os

from setuptools import setup

IN_A_PACKAGE = False
PKG_INFO = os.path.join(os.path.dirname('__file__'), 'PKG-INFO')
if not os.path.exists(PKG_INFO):
    from autosemver import PROJECT_NAME
    from autosemver.packaging import (
        get_authors,
        get_changelog,
        get_current_version,
    )
    __version__ = get_current_version(project_name=PROJECT_NAME)
else:
    IN_A_PACKAGE = True
    with open(PKG_INFO) as info_fd:
        for line in info_fd.readlines():
            if line.startswith('Version: '):
                __version__ = line.split(' ', 1)[-1]
                break
        else:
            raise ImportError('Unable to find version for autosemver')



if __name__ == '__main__':
    if not IN_A_PACKAGE:
        with open('AUTHORS', 'w') as authors_fd:
            authors_fd.write('\n'.join(get_authors()))

        with open('CHANGELOG', 'w') as changelog_fd:
            changelog_fd.write(get_changelog())

    setup(
        author='David Caro',
        author_email='david@dcaro.es',
        description='Tools to handle automatic semantic versioning in python',
        install_requires=['dulwich'],
        license='GPLv3',
        name='autosemver',
        package_data={'': ['CHANGELOG', 'AUTHORS']},
        packages=['autosemver'],
        url='https://github.com/david-caro/python-autosemver',
        version=__version__,
        entry_points={
            'console_scripts': 'autosemver=autosemver:main',
            'distutils.setup_keywords': [
                'autosemver=autosemver:distutils',
                'bugtracker_url=autosemver:distutils',
            ],
        }
    )
